@@ -31,7 +31,7 @@
 
 DeclarativeMimeData::DeclarativeMimeData()
     : QMimeData(),
-    m_source(0)
+    m_source(nullptr)
 {}
 
 /*!
@@ -49,7 +49,7 @@ DeclarativeMimeData::DeclarativeMimeData()
 */
 DeclarativeMimeData::DeclarativeMimeData(const QMimeData* copy)
     : QMimeData(),
-    m_source(0)
+    m_source(nullptr)
 {
     // Copy the standard MIME data
     foreach(QString format, copy->formats()) {
@@ -99,8 +99,9 @@ QJsonArray DeclarativeMimeData::urls() const
 void DeclarativeMimeData::setUrls(const QJsonArray &urls)
 {
     QList<QUrl> urlList;
+    urlList.reserve(urls.size());
     foreach (const QVariant &varUrl, urls) {
-        urlList << varUrl.value<QUrl>();
+        urlList << varUrl.toUrl();
     }
     QMimeData::setUrls(urlList);
     emit urlsChanged();
