@@ -66,6 +66,9 @@ public:
     const KAboutData *_about;
     QString _rootOnlyMessage;
     QString _quickHelp;
+    QList<QQuickItem *> subPages;
+    int _columnWidth = -1;
+    int currentIndex = 0;
     bool _useRootOnlyMessage : 1;
 
     bool _needsAuthorization : 1;
@@ -89,7 +92,8 @@ ConfigModule::ConfigModule(const KPluginMetaData &metaData, QObject *parent, con
 {
     KAboutData *aboutData = new KAboutData(metaData.pluginId(), metaData.name(), metaData.version(), metaData.description(), KAboutLicense::byKeyword(metaData.license()).key());
 
-    for (auto author : metaData.authors()) {
+    const auto authors = metaData.authors();
+    for (auto& author : authors) {
         aboutData->addAuthor(author.name(), author.task(), author.emailAddress(), author.webAddress(), author.ocsUsername());
     }
     setAboutData(aboutData);
@@ -165,7 +169,7 @@ QQuickItem *ConfigModule::mainUi()
 
     if (!package.filePath("mainscript").isEmpty()) {
         d->_qmlObject->setSource(QUrl::fromLocalFile(package.filePath("mainscript")));
-        d->_qmlObject->engine()->rootContext()->setContextProperty(QStringLiteral("kcm"), this);
+        d->_qmlObject->rootContext()->setContextProperty(QStringLiteral("kcm"), this);
         d->_qmlObject->completeInitialization();
 
         return qobject_cast<QQuickItem *>(d->_qmlObject->rootObject());
@@ -175,6 +179,58 @@ QQuickItem *ConfigModule::mainUi()
     }
 }
 
+void ConfigModule::push(const QString &fileName, const QVariantMap &propertyMap)
+{
+    //ensure main ui is created
+    if (!mainUi()) {
+        return;
+    }
+
+    //TODO: package as member
+    KPackage::Package package = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("KPackage/GenericQML"));
+    package.setDefaultPackageRoot(QStringLiteral("kpackage/kcms"));
+    package.setPath(aboutData()->componentName());
+
+    QObject *object = d->_qmlObject->createObjectFromSource(QUrl::fromLocalFile(package.filePath("ui", fileName)), d->_qmlObject->rootContext());//TODO:propertyMap
+
+    QQuickItem *item = qobject_cast<QQuickItem *>(object);
+    if (!item) {
+        object->deleteLater();
+        return;
+    }
+
+    d->subPages << item;
+    emit pagePushed(item);
+    emit depthChanged(depth());
+    setCurrentIndex(d->currentIndex + 1);
+}
+
+void ConfigModule::push(QQuickItem *item)
+{
+    //ensure main ui is created
+    if (!mainUi()) {
+        return;
+    }
+
+    d->subPages << item;
+    emit pagePushed(item);
+    emit depthChanged(depth());
+    setCurrentIndex(d->currentIndex + 1);
+}
+
+void ConfigModule::pop()
+{
+    if (d->subPages.isEmpty()) {
+        return;
+    }
+    QQuickItem *page = d->subPages.takeLast();
+    emit pageRemoved();
+    emit depthChanged(depth());
+    page->deleteLater();
+
+    setCurrentIndex(qMin(d->currentIndex, depth() - 1));
+}
+
 ConfigModule::Buttons ConfigModule::buttons() const
 {
     return d->_buttons;
@@ -224,6 +280,42 @@ QString ConfigModule::description() const
     return d->_about->shortDescription();
 }
 
+int ConfigModule::columnWidth() const
+{
+    return d->_columnWidth;
+}
+
+void ConfigModule::setColumnWidth(int width)
+{
+    if (d->_columnWidth == width) {
+        return;
+    }
+
+    d->_columnWidth = width;
+    emit columnWidthChanged(width);
+}
+
+int ConfigModule::depth() const
+{
+    return d->subPages.count() + 1;
+}
+
+void ConfigModule::setCurrentIndex(int index)
+{
+    if (index < 0 || index > d->subPages.count() || index == d->currentIndex) {
+        return;
+    }
+
+    d->currentIndex = index;
+
+    emit currentIndexChanged(index);
+}
+
+int ConfigModule::currentIndex() const
+{
+    return d->currentIndex;
+}
+
 void ConfigModule::setAuthActionName(const QString &name)
 {
     if (d->_authActionName == name) {
