@@ -33,6 +33,8 @@
 #include <QObject>
 #include <QtQml>
 
+#include <KPluginMetaData>
+
 class QStringList;
 class KAboutData;
 class QQuickItem;
@@ -43,6 +45,8 @@ namespace KQuickAddons {
 class ConfigModulePrivate;
 
 /**
+ * @class KQuickAddons::ConfigModule configmodule.h KQuickAddons/ConfigModule
+ *
  * The base class for configuration modules.
  *
  * Configuration modules are realized as plugins that are loaded only when
@@ -56,7 +60,7 @@ class ConfigModulePrivate;
  * the module author.
  *
  * To write a config module, you have to create a C++ library
- * and an accompaining QML user interface.
+ * and an accompaning QML user interface.
  * The library must contain a factory function like the following:
  *
  * \code
@@ -134,14 +138,18 @@ class QUICKADDONS_EXPORT ConfigModule : public QObject
 {
     Q_OBJECT
 
-    Q_FLAGS(Buttons)
     Q_PROPERTY(QQuickItem *mainUi READ mainUi CONSTANT)
     Q_PROPERTY(KQuickAddons::ConfigModule::Buttons buttons READ buttons WRITE setButtons NOTIFY buttonsChanged)
     Q_PROPERTY(bool needsSave READ needsSave WRITE setNeedsSave NOTIFY needsSaveChanged)
+    Q_PROPERTY(QString name READ name CONSTANT)
+    Q_PROPERTY(QString description READ description CONSTANT)
     Q_PROPERTY(QString quickHelp READ quickHelp WRITE setQuickHelp NOTIFY quickHelpChanged)
     Q_PROPERTY(QString rootOnlyMessage READ rootOnlyMessage WRITE setRootOnlyMessage NOTIFY rootOnlyMessageChanged)
     Q_PROPERTY(bool useRootOnlyMessage READ useRootOnlyMessage WRITE setUseRootOnlyMessage NOTIFY useRootOnlyMessageChanged)
     Q_PROPERTY(bool needsAuthorization READ needsAuthorization WRITE setNeedsAuthorization NOTIFY needsAuthorizationChanged)
+    Q_PROPERTY(int columnWidth READ columnWidth WRITE setColumnWidth NOTIFY columnWidthChanged)
+    Q_PROPERTY(int depth READ depth NOTIFY depthChanged)
+    Q_PROPERTY(int currentIndex READ currentIndex WRITE setCurrentIndex NOTIFY currentIndexChanged)
 
 public:
 
@@ -158,8 +166,9 @@ public:
         Default = 2,
         Apply = 4
     };
-    Q_ENUMS(Button)
+    Q_ENUM(Button)
     Q_DECLARE_FLAGS(Buttons, Button)
+    Q_FLAG(Buttons)
 
     /**
      * Base class for all KControlModules.
@@ -168,7 +177,15 @@ public:
      *       to any slot.
      * @param aboutData becomes owned by the ConfigModule
      */
-    explicit ConfigModule(const KAboutData *aboutData, QObject *parent = 0, const QVariantList &args = QVariantList());
+    explicit ConfigModule(const KAboutData *aboutData, QObject *parent = nullptr, const QVariantList &args = QVariantList());
+
+    /**
+     * @note do not emit changed signals here, since they are not yet connected
+     *       to any slot.
+     * @param metaData description for the plugin: it will generate a KAboutData from that
+     * @since 5.11
+     */
+    explicit ConfigModule(const KPluginMetaData &metaData, QObject *parent = nullptr, const QVariantList &args = QVariantList());
 
     /**
      * Base class for all KControlModules.
@@ -176,7 +193,7 @@ public:
      * @note do not emit changed signals here, since they are not yet connected
      *       to any slot.
      */
-    explicit ConfigModule(QObject *parent = 0, const QVariantList &args = QVariantList());
+    explicit ConfigModule(QObject *parent = nullptr, const QVariantList &args = QVariantList());
 
     /**
      * Destroys the module.
@@ -360,6 +377,51 @@ public:
      */
     bool needsAuthorization() const;
 
+    /**
+     * @returns the name of the config module
+     * @since 5.41
+     */
+    QString name() const;
+
+    /**
+     * @returns the description of the config module
+     * @since 5.41
+     */
+    QString description() const;
+
+    /**
+     * returns the width the kcm wants in column mode.
+     * If a columnWidth is valid ( > 0 ) and less than the systemsettings' view width,
+     * more than one will be visible at once, and the first page will be a sidebar to the last page pushed.
+     * As default, this is -1 which will make the shell always show only one page at a time.
+     * @since 5.50
+     */
+    int columnWidth() const;
+
+    /**
+     * Sets the column width we want.
+     * @since 5.50
+     */
+    void setColumnWidth(int width);
+
+    /**
+     * @returns how many pages this kcm has.
+     * It is guaranteed to be at least 1 9the main ui) plus how many times a new page has been pushed without pop
+     */
+    int depth() const;
+
+    /**
+     * Sets the current page index this kcm should display
+     * @since 5.53
+     */
+    void setCurrentIndex(int index);
+
+    /**
+     * @returns the index of the page this kcm should display
+     * @since 5.53
+     */
+    int currentIndex() const;
+
     static ConfigModule *qmlAttachedProperties(QObject *object);
 
 public Q_SLOTS:
@@ -402,6 +464,22 @@ public Q_SLOTS:
      */
     virtual void defaults();
 
+    /**
+     * Push a new sub page in the KCM hyerarchy: pages will be seen as a Kirigami PageRow
+     * @since 5.50
+     */
+    void push(const QString &fileName, const QVariantMap &propertyMap = QVariantMap());
+
+    /**
+     * 
+     */
+    void push(QQuickItem *item);
+
+    /**
+     * pop the last page of the KCM hyerarchy
+     * @since 5.50
+     */
+    void pop();
 
 Q_SIGNALS:
 
@@ -449,10 +527,41 @@ Q_SIGNALS:
     void rootOnlyMessageChanged();
 
     /**
-     * Emits this signal whenever the the root only message gets used or discarded.
+     * Emits this signal whenever the root only message gets used or discarded.
      */
     void useRootOnlyMessageChanged();
 
+    /**
+     * Emitted when a new sub page is pushed
+     * @since 5.50
+     */
+    void pagePushed(QQuickItem *page);
+
+    /**
+     * Emitted when a sub page is popped
+     * @since 5.50
+     */
+    //RFC: page argument?
+    void pageRemoved();
+
+    /**
+     * Emitted when the wanted column width of the kcm changes
+     * @since 5.50
+     */
+    void columnWidthChanged(int width);
+
+    /**
+     * Emitted when the current page changed
+     * @since 5.53
+     */
+    void currentIndexChanged(int index);
+
+    /**
+     * Emitted when the number of pages changed
+     * @since 5.53
+     */
+    void depthChanged(int index);
+
 private:
     ConfigModulePrivate *const d;
 };
